package {
	import flash.display.Sprite;
	import flash.display.StageAlign;
	import flash.display.StageScaleMode;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.ui.Keyboard;
	
	public class TreeSimulation extends Sprite
	{
		private var trees:Array;
		private var numTrees:uint = 100;
		
		//focal length
		//How wide or how narrow a view the lens will provide. 
		//Smaller focal length numbers mean wider; larger numbers mean narrower.
		private var fl:Number = 250;
		
		//vanishing point
		private var vpX:Number = stage.stageWidth / 2;
		private var vpY:Number = stage.stageHeight / 2;
		
		//how high the floor is
		private var floor:Number = 50;
		
		//acceleration
		private var ax:Number = 0;
		private var ay:Number = 0;
		private var az:Number = 0;
		
		//velocities
		private var vx:Number = 0;
		private var vy:Number = 0;
		private var vz:Number = 0;
		
		//gravity
		private var gravity:Number = 0.3;
		
		//how much to dampen velocity effects
		private var friction:Number = 0.98;
		
		public function TreeSimulation()
		{
			init();
		}
		
		private function init():void
		{	
			stage.align = StageAlign.TOP_LEFT;
			stage.scaleMode = StageScaleMode.NO_SCALE;
			
			trees = new Array();
			
			for(var i:uint = 0; i < numTrees; i++)
			{
				var tree:Tree = new Tree();
				trees.push(tree);
				
				//set the 3d positions of the tree
				tree.xpos = Math.random() * 2000 - 1000;
				tree.ypos = floor;
				tree.zpos = Math.random() * 10000;
				addChild(tree);
			}
			addEventListener(Event.ENTER_FRAME, onEnterFrame);
			stage.addEventListener(KeyboardEvent.KEY_DOWN, onKeyDown);
			stage.addEventListener(KeyboardEvent.KEY_UP, onKeyUp);
		}
		
		private function onEnterFrame(event:Event):void
		{	
			//add the acceleration to the velocities
			vx += ax;
			vy += ay;
			vz += az;
			
			//affect the y with gravity
			vy -= gravity;
			for(var i:uint = 0; i < numTrees; i++)
			{
				var tree:Tree = trees[i];
				move(tree);
			}
			
			//apply friction
			vx *= friction;
			vy *= friction;
			vz *= friction;
			
			//sort z positions
			sortZ();
		}
		
		private function onKeyDown(event:KeyboardEvent):void
		{	
			
			//determine which key is pressed
			switch(event.keyCode)
			{
				case Keyboard.UP :
				az = -1;
				break;
				
				case Keyboard.DOWN :
				az = 1;
				break;
				
				case Keyboard.LEFT :
				ax = 1;
				break;
				
				case Keyboard.RIGHT :
				ax = -1;
				break;
				
				case Keyboard.SPACE :
				ay = 1;
				break;
				
				default :
				break;
			}
		}
		
		private function onKeyUp(event:KeyboardEvent):void
		{	
			//determines which key was released
			switch(event.keyCode)
			{
				case Keyboard.UP :
				case Keyboard.DOWN :
				az = 0;
				break;
				
				case Keyboard.LEFT :
				case Keyboard.RIGHT :
				ax = 0;
				break;
				
				case Keyboard.SPACE :
				ay = 0;
				break;
				
				default :
				break;
			}
		}
		
		private function move(tree:Tree):void
		{	
			
			//moves a tree
			tree.xpos += vx;
			tree.ypos += vy;
			tree.zpos += vz;
			
			//if the tree's y position is less than the floor, 
			//set it to the floor
			if(tree.ypos < floor)
			{
				tree.ypos = floor;
			}
			
			//if the zposition is less than the focal length...
			if(tree.zpos < -fl)
			{
				//pusth the tree 10000 z units front
				tree.zpos += 10000;
			}
			
			//if from the above operation the zpos is greater than 1000 minus the
			//focal length subtract 10000 - fl
			if(tree.zpos > 10000 - fl)
			{
				tree.zpos -= 10000;
			}
			
			//create a ratio used for scaling
			var scale:Number = fl / (fl + tree.zpos);
			tree.scaleX = tree.scaleY = scale;
			
			//the positions based on the vanishing points and the offset
			tree.x = vpX + tree.xpos * scale;
			tree.y = vpY + tree.ypos * scale;
			
			//do alpha transform based on the scale
			tree.alpha = scale;
		}
		
		private function sortZ():void
		{
			//sort based on the z positions
			trees.sortOn("zpos", Array.DESCENDING | Array.NUMERIC);
			for(var i:uint = 0; i < numTrees; i++)
			{
				var tree:Tree = trees[i];
				setChildIndex(tree, i);
			}
		}
	}
}
