package {
	import flash.display.Sprite;
	import flash.display.StageAlign;
	import flash.display.StageScaleMode;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.ui.Keyboard;
	import flash.xml.*;
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	
	
	public class gallery extends Sprite {
		var xmlLoader:URLLoader;
		var xmlData:XML;
		
		//this holds the thumbs
		private var thumbs:Array;
		
		//directories of images
		private var thumbdir:String;
		private var fulldir:String;
		
		//focal length
		//How wide or how narrow a view the lens will provide. 
		//Smaller focal length numbers mean wider; larger numbers mean narrower.
		private var fl:Number = 250;
		
		//vanishing point
		private var vpX:Number = stage.stageWidth / 2;
		private var vpY:Number = stage.stageHeight / 2;
		
		//z-spacing
		private var zspacing:Number = 17000;
		
		//how high the floor is
		private var floor:Number = 230;
		
		//acceleration
		private var ax:Number = 0;
		private var ay:Number = 0;
		private var az:Number = 0;
		
		//velocities
		private var vx:Number = 0;
		private var vy:Number = 0;
		private var vz:Number = 0;
		
		//gravity
		private var gravity:Number = 0.3;
		
		//how much to dampen velocity effects
		private var friction:Number = 0.98;
		
		public function gallery() { //runs on create
			init();
		}
		
		private function init():void { //i can has privacy?
			
			stage.align = StageAlign.TOP_LEFT;
			stage.scaleMode = StageScaleMode.NO_SCALE;
			
			//there is a statusText dynamic text already on the stage
			this.statusText.text = 'Loading thumbnails...';
			
			//load the xml
			xmlLoader = new URLLoader();
			xmlLoader.addEventListener(Event.COMPLETE, xmlLoadHandler);
			xmlLoader.load(new URLRequest('./pictures.xml'));			
		}
		
		function xmlLoadHandler(e:Event):void {
			xmlData = new XML(e.target.data);
			//thumbdir
			//fulldir
			var xmlItemList:XMLList = xmlData.picture;
				//here we are asking for all the subelements called "picture"
			this.statusText.text = 'Creating thumbnails...';
			createThumbnails(xmlItemList);
		}
		
		function createThumbnails(xmlData:XMLList):void { //blorp
			//here copied stuff from the original init
			thumbs = new Array();

			for(var i:uint; i < xmlData.length(); i++) {
				var url = './thumbs/'+xmlData[i].attribute('filename');
				var desc = String(xmlData[i]); //is this the right way?
				var w = xmlData[i].attribute('width');
				var h = xmlData[i].attribute('height');
				var thumb:thumbnail = new thumbnail(url,desc,w,h);
				//that's thumbnail(url,desc,width,height);
				thumbs.push(thumb);
								
				//set the 3d positions of the thumb
				thumb.xpos = Math.random() * 2000 - 1000;
				thumb.ypos = floor;
				thumb.zpos = zspacing*(i/xmlData.length());
				addChild(thumb);
			}
			addEventListener(Event.ENTER_FRAME, onEnterFrame);
			stage.addEventListener(KeyboardEvent.KEY_DOWN, onKeyDown);
			stage.addEventListener(KeyboardEvent.KEY_UP, onKeyUp);
			
			//update the status
			this.statusText.text = 'Use arrow keys to move around; hold spacebar to fly.';
			//this.statusText.text = xmlData.length();
		}
		
		/*function clickHandler(e:MouseEvent):void { //blorp
			var target = e.currentTarget;
			loadImage(target.img);
			imageTexty.titleText.text = target.titletext;
			imageTexty.descriptionText.text = target.desctext;
			if(target.titletext=='Orange'||target.titletext=='Silver'){
				imageTexty.x = imageTextyX + 60;
			} else if(target.titletext=='Copper'||target.titletext=='Green') {
				imageTexty.x = imageTextyX - 30;
			} else {
				imageTexty.x = imageTextyX;
			}
		}*/
		
		private function onEnterFrame(event:Event):void
		{	
			//add the acceleration to the velocities
			vx += ax;
			vy += ay;
			vz += az;
			
			//affect the y with gravity
			vy -= gravity;
			for(var i:uint = 0; i < thumbs.length; i++)
			{
				var thumb:thumbnail = thumbs[i];
				move(thumb);
			}
			
			//apply friction
			vx *= friction;
			vy *= friction;
			vz *= friction;
			
			//sort z positions
			sortZ();
		}
		
		private function onKeyDown(event:KeyboardEvent):void
		{	
			this.statusText.alpha = 0; //hide this
			
			//determine which key is pressed
			switch(event.keyCode)
			{
				case Keyboard.UP :
				az = -1;
				break;
				
				case Keyboard.DOWN :
				az = 1;
				break;
				
				case Keyboard.LEFT :
				ax = 1;
				break;
				
				case Keyboard.RIGHT :
				ax = -1;
				break;
				
				case Keyboard.SPACE :
				ay = 1;
				break;
				
				default :
				break;
			}
		}
		
		private function onKeyUp(event:KeyboardEvent):void
		{	
			//determines which key was released
			switch(event.keyCode)
			{
				case Keyboard.UP :
				case Keyboard.DOWN :
				az = 0;
				break;
				
				case Keyboard.LEFT :
				case Keyboard.RIGHT :
				ax = 0;
				break;
				
				case Keyboard.SPACE :
				ay = 0;
				break;
				
				default :
				break;
			}
		}
		
		private function move(thumb:thumbnail):void
		{	
			
			//moves a thumb
			thumb.xpos += vx;
			thumb.ypos += vy;
			thumb.zpos += vz;
			
			//if the thumb's y position is less than the floor, 
			//set it to the floor
			if(thumb.ypos < floor)
			{
				thumb.ypos = floor;
			}
			
			//if the zposition is less than the focal length...
			if(thumb.zpos < -fl)
			{
				//pusth the thumb 10000 z units front
				thumb.zpos += zspacing;
			}
			
			//if from the above operation the zpos is greater than 1000 minus the
			//focal length subtract 10000 - fl
			if(thumb.zpos > zspacing - fl)
			{
				thumb.zpos -= zspacing;
			}
			
			//create a ratio used for scaling
			var scale:Number = fl / (fl + thumb.zpos);
			thumb.scaleX = thumb.scaleY = scale;
			
			//the positions based on the vanishing points and the offset
			thumb.x = vpX + thumb.xpos * scale;
			thumb.y = vpY + thumb.ypos * scale;
			
			//do alpha transform based on the scale
			thumb.alpha = ((scale-0.03)*6); //so that they will be "opaque" more near the front
		}
		
		private function sortZ():void
		{
			//sort based on the z positions
			thumbs.sortOn("zpos", Array.DESCENDING | Array.NUMERIC);
			for(var i:uint = 0; i < thumbs.length; i++)
			{
				var thumb:thumbnail = thumbs[i];
				setChildIndex(thumb, i);
			}
		}
	}
}
