//BY   : Ed Salvana
//WEB  : ed.visualgratis.com
//BLOG : visualgratis.com
//MAIL : ed@visualgratis.com
//DATE : 02.19.09
//WHAT : Using Perlin noise to create a displacement map to simulate waves. Nt
//		 exactly accurate, but passable.
//ETC  : Bits of this were referenced from the book Learning ActionScript 3
//		 ( www.learningactionscript3.com ), a pretty damn good useful book.





package {

	import flash.display.*;
	import flash.utils.Timer;
	import flash.events.*;
	import flash.text.TextField;
	import flash.geom.Point;
	import flash.filters.DisplacementMapFilter;
	import flash.filters.DisplacementMapFilterMode;
	
	public class DisplacementWaves extends Sprite {
		
		//create a bitmap data object to create our perlin noise in
		private var bmpData:BitmapData = new BitmapData( stage.stageWidth, stage.stageHeight );

		//setup perlin noise settings
		//still figuring out some of these, but tweaking the will affect how the waves look
		private var baseX:Number = 50;
		private var baseY:Number = 30;
		private var octaves:Number = 5;
		private var seed:Number = 1;
		private var stitch:Boolean = true;
		private var fractal:Boolean = true;
		private var channelOptions:Number = BitmapDataChannel.BLUE;
		private var greyScale:Boolean = false;
		private var offsets:Array = new Array( new Point(), new Point() );

		//setup displacement map settings
		//same as the perlin noise
		private var dMap:DisplacementMapFilter;
		private var xScale:Number = 2;
		private var yScale:Number = 4;
		private var componentX:uint = BitmapDataChannel.BLUE; 
		private var componentY:uint = BitmapDataChannel.BLUE; 
		private var dMode:String = DisplacementMapFilterMode.WRAP;

		//create a timer for offsetting the noise
		private var timer:Timer = new Timer( 30 );
					
		public function DisplacementWaves()
		{
			init();
		}
	

		function init():void{

			//event listener for the timer
			timer.addEventListener("timer", updateNoise );

			//handles start/stopping the timer
			addEventListener(MouseEvent.CLICK, toggleWaves );

			//shows hand cursor
			buttonMode = true;

			//links the displacement map to the bitmap data (making it the source)
			dMap = new DisplacementMapFilter( bmpData, new Point(), componentX, componentY, xScale, yScale, dMode );

			timer.start();
		}


		function updateNoise( e:TimerEvent ){

			//shifts the offsets, this really is the key to having the perlin noise shift seamlessly
			offsets[0].x += Math.random()*1 + 1;
			offsets[0].y += Math.random()*1 + 1;
			offsets[1].x += Math.random()*1 + 1;
			offsets[1].y += Math.random()*1 + 1;

			//we modify the bitmap data, mostly with the offsets and a little with the baseX & baseY
			bmpData.perlinNoise( baseX + ( Math.random()*1), baseY + ( Math.random()*1 ), octaves, seed, stitch, fractal, channelOptions, greyScale, offsets );

			//apply the displacement map
			image.filters = [  dMap ];

		}

		function toggleWaves( e:MouseEvent ):void{

			//pretty self explanatory
			if( timer.running ){
				timer.stop();
				tfStatus.text = "PAUSED";
			} else {
				timer.start();
				tfStatus.text = "PLAYING";
			}
		}


		
	}

}

